@@ -62,11 +62,11 @@ static uid_t getpiduid(const pid_t pid);
 static int print_matches(struct names *names_head, const opt_type opts, const int sig_number);
 static void kill_matched_proc(struct procs *pptr, const opt_type opts, const int sig_number);
 
-static dev_t get_netdev(void);
 int parse_mount(struct names *this_name, struct device_list **dev_list);
 static void add_device(struct device_list **dev_list, struct names  *this_name, dev_t device);
 void scan_mount_devices(const opt_type opts, struct mountdev_list **mount_devices);
-void scan_unixsockets(struct unixsocket_list **unixsocket_head);
+void fill_unix_cache(struct unixsocket_list **unixsocket_head);
+static dev_t find_net_dev(void);
 #ifdef DEBUG
 static void debug_match_lists(struct names *names_head, struct inode_list *ino_head, struct device_list *dev_head);
 #endif
@@ -334,11 +334,21 @@ int parse_file(struct names *this_name, struct inode_list **ino_list)
 int parse_unixsockets(struct names *this_name, struct inode_list **ino_list, struct unixsocket_list *sun_head)
 {
 	struct unixsocket_list *sun_tmp;
+	struct stat st;
+    dev_t net_dev;
+    
+	if (stat(this_name->filename, &st) != 0) {
+		fprintf(stderr,_("Cannot stat %s: %s\n"), this_name->filename,
+				strerror(errno));
+		return -1;
+	}
+    net_dev = find_net_dev();
 
 	for (sun_tmp = sun_head; sun_tmp != NULL ; sun_tmp = sun_tmp->next)
 	{
-		if (strcmp(this_name->filename, sun_tmp->sun_name) == 0) {
-			add_inode(ino_list, this_name, sun_tmp->dev, sun_tmp->inode);
+      if (sun_tmp->dev == st.st_dev && sun_tmp->inode == st.st_ino) {
+			add_inode(ino_list, this_name, net_dev, sun_tmp->net_inode);
+            return 0;
 		}
 	}
 	return 0;
@@ -519,7 +529,7 @@ void find_net_sockets(struct inode_list **ino_list, struct ip_connections *conn_
 		return ;
 
 	if ( (fp = fopen(pathname, "r")) == NULL) {
-		fprintf(stderr, _("Cannot open protocol file \"%s\": %s"), pathname,strerror(errno));
+		fprintf(stderr, _("Cannot open protocol file \"%s\": %s\n"), pathname,strerror(errno));
 		return;
 	}
 	while (fgets(line, BUFSIZ, fp) != NULL) {
@@ -593,7 +603,7 @@ void find_net6_sockets(struct inode_list **ino_list, struct ip6_connections *con
 		/*printf("Found %ld with %s:%ld\n", loc_port, rmt_addr6str, rmt_port);*/
 		for(conn_tmp = conn_list ; conn_tmp != NULL ; conn_tmp = conn_tmp->next) {
 			inet_ntop(AF_INET6, &conn_tmp->rmt_address, rmt_addr6str, INET6_ADDRSTRLEN);
-		/*	printf("Comparing with *.%lu %s:%lu ...", 
+			/*printf("Comparing with *.%lu %s:%lu ...\n\n", 
 					conn_tmp->lcl_port,
 					rmt_addr6str,
 					conn_tmp->rmt_port);*/
@@ -640,9 +650,9 @@ int main(int argc, char *argv[])
 	opts = 0;
 	sig_number = SIGKILL;
 
-	netdev = get_netdev();
+	netdev = find_net_dev();
 	scan_mount_devices(opts, &mount_devices);
-	scan_unixsockets(&unixsockets);
+	fill_unix_cache(&unixsockets);
 
 	/* getopt doesnt like things like -SIGBLAH */
 	for(optc = 1; optc < argc; optc++) {
@@ -655,7 +665,8 @@ int main(int argc, char *argv[])
 				/* FIXME longopts */
 				continue;
 			}
-			while (*option) switch(*option++) {
+			while (*option) {
+				switch(*option) {
 #ifdef WITH_IPV6
 				case '4':
 					ipv4_only = 1;
@@ -717,13 +728,17 @@ int main(int argc, char *argv[])
 				default:
 					if ( isupper(*option) || isdigit(*option) ) {
 						sig_number = get_signal(option,"fuser");
+						option[1]='\0';
+
 						break;
 					}
 					fprintf(stderr,"%s: Invalid option %c\n",argv[0] , argv[optc][1]);
-
 					usage(NULL);
+
 					break;
-			} /* switch */
+				} /* switch */
+				option++;
+			}
 			continue;
 		}
 		/* File specifications */
@@ -833,79 +848,81 @@ static int print_matches(struct names *names_head, const opt_type opts, const in
 	struct passwd *pwent = NULL;
 	int have_match = 0;
 	
-	for (nptr = names_head; nptr != NULL ; nptr = nptr->next) {
-		if (nptr->matched_procs != NULL || opts & OPT_ALLFILES) {
-			if (head == 0 && opts & OPT_VERBOSE) {
-				fprintf(stderr, _("\n%*s USER        PID ACCESS COMMAND\n"),
-				        NAME_FIELD, "");
-				head = 1;
-			}
-
-			fprintf(stderr, "%s:", nptr->filename);
-			len = strlen(nptr->filename) + 1;
-		}
-
-		first = 1;
-		for (pptr = nptr->matched_procs; pptr != NULL ; pptr = pptr->next) {
-			have_match = 1;
-			if (opts & (OPT_VERBOSE|OPT_USER)) {
-				if (pwent == NULL || pwent->pw_uid != pptr->uid)
-					pwent = getpwuid(pptr->uid);
-			}
-			if (len > NAME_FIELD && (opts & OPT_VERBOSE)) {
-				putc('\n', stderr);
-				len=0;
-			}
-			if ((opts & OPT_VERBOSE) || first) 
-				while (len++ < NAME_FIELD)
-					putc(' ', stderr);
-			if (opts & OPT_VERBOSE) {
-				if (pwent == NULL)
-					fprintf(stderr, " %-8s ", _("(unknown)"));
-				else
-					fprintf(stderr, " %-8s ", pwent->pw_name);
-			}
-			printf("%6d", pptr->pid);
-			fflush(stdout);
-			if (opts & OPT_VERBOSE) {
-				fprintf(stderr, " %c%c%c%c%c ",
-						pptr->access & ACCESS_FILE ? (pptr->access & ACCESS_FILEWR ? 'F' : 'f' ) : '.',
-						pptr->access & ACCESS_ROOT ? 'r' : '.',
-						pptr->access & ACCESS_CWD ? 'c' : '.',
-						pptr->access & ACCESS_EXE ? 'e' : '.',
-						(pptr->access & ACCESS_MMAP) && !(pptr->access & ACCESS_EXE) ? 'm' : '.');
-			} else {
-				if (pptr->access & ACCESS_ROOT)
-					putc('r', stderr);
-				if (pptr->access & ACCESS_CWD)
-					putc('c', stderr);
-				if (pptr->access & ACCESS_EXE)
-					putc('e', stderr);
-				else if (pptr->access & ACCESS_MMAP)
-					putc('m', stderr);
-			}
-			if (opts & OPT_USER) {
-				if (pwent == NULL)
-					fprintf(stderr, " %-8s ", _("(unknown)"));
-				else
-					fprintf(stderr, "(%s)", pwent->pw_name);
-			}
-			if (opts & OPT_VERBOSE) {
-				if (pptr->command == NULL)
-					fprintf(stderr, "???\n");
-				else 
-					fprintf(stderr, "%s\n", pptr->command);
-			}
-			len = 0;
-			first = 0;
-		}
-		if (nptr->matched_procs != NULL || opts & OPT_ALLFILES)
-			putc('\n', stderr);
+    	for (nptr = names_head; nptr != NULL ; nptr = nptr->next) {
+		if (! (opts & OPT_SILENT)) { /* We're not silent */
+    			if (nptr->matched_procs != NULL || opts & OPT_ALLFILES) {
+    				if (head == 0 && opts & OPT_VERBOSE) {
+    					fprintf(stderr, _("\n%*s USER        PID ACCESS COMMAND\n"),
+    				        	NAME_FIELD, "");
+    					head = 1;
+    				}
+    
+    				fprintf(stderr, "%s:", nptr->filename);
+    				len = strlen(nptr->filename) + 1;
+    			}
+    
+    			first = 1;
+    			for (pptr = nptr->matched_procs; pptr != NULL ; pptr = pptr->next) {
+    				have_match = 1;
+    				if (opts & (OPT_VERBOSE|OPT_USER)) {
+    					if (pwent == NULL || pwent->pw_uid != pptr->uid)
+    						pwent = getpwuid(pptr->uid);
+    				}
+    				if (len > NAME_FIELD && (opts & OPT_VERBOSE)) {
+    					putc('\n', stderr);
+    					len=0;
+    				}
+    				if ((opts & OPT_VERBOSE) || first) 
+    					while (len++ < NAME_FIELD)
+    						putc(' ', stderr);
+    				if (opts & OPT_VERBOSE) {
+    					if (pwent == NULL)
+    						fprintf(stderr, " %-8s ", _("(unknown)"));
+    					else
+    						fprintf(stderr, " %-8s ", pwent->pw_name);
+    				}
+    				printf("%6d", pptr->pid);
+    				fflush(stdout);
+    				if (opts & OPT_VERBOSE) {
+    					fprintf(stderr, " %c%c%c%c%c ",
+    						pptr->access & ACCESS_FILE ? (pptr->access & ACCESS_FILEWR ? 'F' : 'f' ) : '.',
+    						pptr->access & ACCESS_ROOT ? 'r' : '.',
+    						pptr->access & ACCESS_CWD ? 'c' : '.',
+    						pptr->access & ACCESS_EXE ? 'e' : '.',
+    						(pptr->access & ACCESS_MMAP) && !(pptr->access & ACCESS_EXE) ? 'm' : '.');
+    				} else {
+    					if (pptr->access & ACCESS_ROOT)
+    						putc('r', stderr);
+    					if (pptr->access & ACCESS_CWD)
+    						putc('c', stderr);
+    					if (pptr->access & ACCESS_EXE)
+    						putc('e', stderr);
+    					else if (pptr->access & ACCESS_MMAP)
+    						putc('m', stderr);
+    				}
+    				if (opts & OPT_USER) {
+    					if (pwent == NULL)
+    						fprintf(stderr, " %-8s ", _("(unknown)"));
+    					else
+    						fprintf(stderr, "(%s)", pwent->pw_name);
+    				}
+    				if (opts & OPT_VERBOSE) {
+    					if (pptr->command == NULL)
+    						fprintf(stderr, "???\n");
+    					else 
+    						fprintf(stderr, "%s\n", pptr->command);
+    				}
+    				len = 0;
+    				first = 0;
+    			}
+    			if (nptr->matched_procs != NULL || opts & OPT_ALLFILES)
+    				putc('\n', stderr);
+		} /* be silent */
 		if (opts & OPT_KILL)
-			kill_matched_proc(nptr->matched_procs,  opts, sig_number);
+		kill_matched_proc(nptr->matched_procs,  opts, sig_number);
 
 	} /* next name */
-	return !have_match;
+	return (have_match==1?0:1);
 
 }
 
@@ -1028,10 +1045,10 @@ void add_mount_device(struct mountdev_list **mount_head,const char *fsname, cons
 }
 
 /*
- * scan_unixsockets : Create a list of Unix sockets
+ * fill_unix_cache : Create a list of Unix sockets
  *   This list is used later for matching purposes
  */
-void scan_unixsockets(struct unixsocket_list **unixsocket_head)
+void fill_unix_cache(struct unixsocket_list **unixsocket_head)
 {
 	FILE *fp;
 	char line[BUFSIZ];
@@ -1057,8 +1074,9 @@ void scan_unixsockets(struct unixsocket_list **unixsocket_head)
 		if ( (newsocket = malloc(sizeof(struct unixsocket_list))) == NULL)
 			continue;
 		newsocket->sun_name = strdup(scanned_path);
-		newsocket->inode = scanned_inode; /* st.st_ino;*/
-		newsocket->dev = 0; /* st.st_dev;*/
+		newsocket->inode = st.st_ino;
+		newsocket->dev = st.st_dev;
+        newsocket->net_inode = scanned_inode;
 		newsocket->next = *unixsocket_head;
 		*unixsocket_head = newsocket;
 	} /* while */
@@ -1087,18 +1105,6 @@ void scan_mount_devices(const opt_type opts, struct mountdev_list **mount_device
 	}
 }
 
-static dev_t get_netdev(void)
-{
-	int skt;
-	struct stat st;
-
-	if ( (skt = socket(PF_INET,SOCK_DGRAM,0)) < 0)
-		return -1;
-	if ( fstat(skt, &st) != 0) 
-		return -1;
-	return st.st_dev;
-}
-
 #ifdef DEBUG
 /* often not used, doesnt need translation */
 static void debug_match_lists(struct names *names_head, struct inode_list *ino_head, struct device_list *dev_head)
@@ -1160,6 +1166,32 @@ static void kill_matched_proc(struct procs *proc_head, const opt_type opts, cons
 	for (pptr = proc_head ; pptr != NULL ; pptr = pptr->next ) {
 		if ( (opts & OPT_INTERACTIVE) && (ask(pptr->pid) == 0))
 			continue;
-		kill (pptr->pid, sig_number);
+		if ( kill (pptr->pid, sig_number) < 0) {
+			fprintf(stderr, _("Could not kill process %d: %s\n"),
+					pptr->pid,
+					strerror(errno)
+				);
+		}
+					
 	}
 }
+
+static dev_t find_net_dev(void)
+{
+  int skt;
+  struct stat st;
+
+  if ( (skt = socket(PF_INET, SOCK_DGRAM, 0)) < 0) {
+    fprintf(stderr,_("Cannot open a network socket.\n"));
+    return -1;
+  }
+  if ( fstat(skt, &st) != 0) {
+    fprintf(stderr,_("Cannot find socket's device number.\n"));
+    close(skt);
+    return -1;
+  }
+  close(skt);
+  return st.st_dev;
+}
+
+
